@@ -1,3 +1,16 @@
+#  Licensed under the Apache License, Version 2.0 (the "License"); you may
+#  not use this file except in compliance with the License. You may obtain
+#  a copy of the License at
+#
+#       http://www.apache.org/licenses/LICENSE-2.0
+#
+#  Unless required by applicable law or agreed to in writing, software
+#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
+#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
+#  License for the specific language governing permissions and limitations
+#  under the License.
+
+from .exception import NoMatches, MultipleMatches
 from .named import NamedExtensionManager
 
 
@@ -20,19 +33,23 @@ class DriverManager(NamedExtensionManager):
         is True.
     :type invoke_kwds: dict
     :param on_load_failure_callback: Callback function that will be called when
-        a entrypoint can not be loaded. The arguments that will be provided
+        an entrypoint can not be loaded. The arguments that will be provided
         when this is called (when an entrypoint fails to load) are
         (manager, entrypoint, exception)
     :type on_load_failure_callback: function
     :param verify_requirements: Use setuptools to enforce the
         dependencies of the plugin(s) being loaded. Defaults to False.
     :type verify_requirements: bool
+    :type warn_on_missing_entrypoint: bool
     """
 
     def __init__(self, namespace, name,
                  invoke_on_load=False, invoke_args=(), invoke_kwds={},
                  on_load_failure_callback=None,
-                 verify_requirements=False):
+                 verify_requirements=False,
+                 warn_on_missing_entrypoint=True):
+        on_load_failure_callback = on_load_failure_callback \
+            or self._default_on_load_failure
         super(DriverManager, self).__init__(
             namespace=namespace,
             names=[name],
@@ -41,8 +58,13 @@ class DriverManager(NamedExtensionManager):
             invoke_kwds=invoke_kwds,
             on_load_failure_callback=on_load_failure_callback,
             verify_requirements=verify_requirements,
+            warn_on_missing_entrypoint=warn_on_missing_entrypoint
         )
 
+    @staticmethod
+    def _default_on_load_failure(drivermanager, ep, err):
+        raise
+
     @classmethod
     def make_test_instance(cls, extension, namespace='TESTING',
                            propagate_map_exceptions=False,
@@ -63,7 +85,7 @@ class DriverManager(NamedExtensionManager):
             and then ignored
         :type propagate_map_exceptions: bool
         :param on_load_failure_callback: Callback function that will
-            be called when a entrypoint can not be loaded. The
+            be called when an entrypoint can not be loaded. The
             arguments that will be provided when this is called (when
             an entrypoint fails to load) are (manager, entrypoint,
             exception)
@@ -87,14 +109,14 @@ class DriverManager(NamedExtensionManager):
 
         if not self.extensions:
             name = self._names[0]
-            raise RuntimeError('No %r driver found, looking for %r' %
-                               (self.namespace, name))
+            raise NoMatches('No %r driver found, looking for %r' %
+                            (self.namespace, name))
         if len(self.extensions) > 1:
             discovered_drivers = ','.join(e.entry_point_target
                                           for e in self.extensions)
 
-            raise RuntimeError('Multiple %r drivers found: %s' %
-                               (self.namespace, discovered_drivers))
+            raise MultipleMatches('Multiple %r drivers found: %s' %
+                                  (self.namespace, discovered_drivers))
 
     def __call__(self, func, *args, **kwds):
         """Invokes func() for the single loaded extension.
