@@ -1,10 +1,23 @@
+#  Licensed under the Apache License, Version 2.0 (the "License"); you may
+#  not use this file except in compliance with the License. You may obtain
+#  a copy of the License at
+#
+#       http://www.apache.org/licenses/LICENSE-2.0
+#
+#  Unless required by applicable law or agreed to in writing, software
+#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
+#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
+#  License for the specific language governing permissions and limitations
+#  under the License.
+
 """ExtensionManager
 """
 
-import pkg_resources
-
+import operator
 import logging
 
+from . import _cache
+from .exception import NoMatches
 
 LOG = logging.getLogger(__name__)
 
@@ -20,7 +33,7 @@ class Extension(object):
     :param name: The entry point name.
     :type name: str
     :param entry_point: The EntryPoint instance returned by
-        :mod:`pkg_resources`.
+        :mod:`entrypoints`.
     :type entry_point: EntryPoint
     :param plugin: The value returned by entry_point.load()
     :param obj: The object returned by ``plugin(*args, **kwds)`` if the
@@ -34,6 +47,38 @@ class Extension(object):
         self.plugin = plugin
         self.obj = obj
 
+    @property
+    def module_name(self):
+        """The name of the module from which the entry point is loaded.
+
+        :return: A string in 'dotted.module' format.
+        """
+        # NOTE: importlib_metadata from PyPI includes this but the
+        # Python 3.8 standard library does not.
+        match = self.entry_point.pattern.match(self.entry_point.value)
+        return match.group('module')
+
+    @property
+    def extras(self):
+        """The 'extras' settings for the plugin."""
+        # NOTE: The underlying package returns re.Match objects for
+        # some reason. Translate those to the matched strings, which
+        # seem more useful.
+        return [
+            # Python 3.6 returns _sre.SRE_Match objects. Later
+            # versions of python return re.Match objects. Both types
+            # have a 'string' attribute containing the text that
+            # matched the pattern.
+            getattr(e, 'string', e)
+            for e in self.entry_point.extras
+        ]
+
+    @property
+    def attr(self):
+        """The attribute of the module to be loaded."""
+        match = self.entry_point.pattern.match(self.entry_point.value)
+        return match.group('attr')
+
     @property
     def entry_point_target(self):
         """The module and attribute referenced by this extension's entry_point.
@@ -41,8 +86,7 @@ class Extension(object):
         :return: A string representation of the target of the entry point in
             'dotted.module:object' format.
         """
-        return '%s:%s' % (self.entry_point.module_name,
-                          self.entry_point.attrs[0])
+        return self.entry_point.value
 
 
 class ExtensionManager(object):
@@ -66,7 +110,7 @@ class ExtensionManager(object):
         then ignored
     :type propagate_map_exceptions: bool
     :param on_load_failure_callback: Callback function that will be called when
-        a entrypoint can not be loaded. The arguments that will be provided
+        an entrypoint can not be loaded. The arguments that will be provided
         when this is called (when an entrypoint fails to load) are
         (manager, entrypoint, exception)
     :type on_load_failure_callback: function
@@ -112,7 +156,7 @@ class ExtensionManager(object):
             are logged and then ignored
         :type propagate_map_exceptions: bool
         :param on_load_failure_callback: Callback function that will
-            be called when a entrypoint can not be loaded. The
+            be called when an entrypoint can not be loaded. The
             arguments that will be provided when this is called (when
             an entrypoint fails to load) are (manager, entrypoint,
             exception)
@@ -139,20 +183,39 @@ class ExtensionManager(object):
 
     def _init_plugins(self, extensions):
         self.extensions = extensions
-        self._extensions_by_name = None
+        self._extensions_by_name_cache = None
+
+    @property
+    def _extensions_by_name(self):
+        if self._extensions_by_name_cache is None:
+            d = {}
+            for e in self.extensions:
+                d[e.name] = e
+            self._extensions_by_name_cache = d
+        return self._extensions_by_name_cache
 
     ENTRY_POINT_CACHE = {}
 
-    def _find_entry_points(self, namespace):
-        if namespace not in self.ENTRY_POINT_CACHE:
-            eps = list(pkg_resources.iter_entry_points(namespace))
-            self.ENTRY_POINT_CACHE[namespace] = eps
-        return self.ENTRY_POINT_CACHE[namespace]
+    def list_entry_points(self):
+        """Return the list of entry points for this namespace.
+
+        The entry points are not actually loaded, their list is just read and
+        returned.
+
+        """
+        if self.namespace not in self.ENTRY_POINT_CACHE:
+            eps = list(_cache.get_group_all(self.namespace))
+            self.ENTRY_POINT_CACHE[self.namespace] = eps
+        return self.ENTRY_POINT_CACHE[self.namespace]
+
+    def entry_points_names(self):
+        """Return the list of entry points names for this namespace."""
+        return list(map(operator.attrgetter("name"), self.list_entry_points()))
 
     def _load_plugins(self, invoke_on_load, invoke_args, invoke_kwds,
                       verify_requirements):
         extensions = []
-        for ep in self._find_entry_points(self.namespace):
+        for ep in self.list_entry_points():
             LOG.debug('found extension %r', ep)
             try:
                 ext = self._load_one_plugin(ep,
@@ -166,15 +229,30 @@ class ExtensionManager(object):
             except (KeyboardInterrupt, AssertionError):
                 raise
             except Exception as err:
-                LOG.error('Could not load %r: %s', ep.name, err)
-                LOG.exception(err)
                 if self._on_load_failure_callback is not None:
                     self._on_load_failure_callback(self, ep, err)
+                else:
+                    # Log the reason we couldn't import the module,
+                    # usually without a traceback. The most common
+                    # reason is an ImportError due to a missing
+                    # dependency, and the error message should be
+                    # enough to debug that.  If debug logging is
+                    # enabled for our logger, provide the full
+                    # traceback.
+                    LOG.error('Could not load %r: %s', ep.name, err,
+                              exc_info=LOG.isEnabledFor(logging.DEBUG))
         return extensions
 
     def _load_one_plugin(self, ep, invoke_on_load, invoke_args, invoke_kwds,
                          verify_requirements):
-        plugin = ep.load(require=verify_requirements)
+        # NOTE(dhellmann): Using require=False is deprecated in
+        # setuptools 11.3.
+        if hasattr(ep, 'resolve') and hasattr(ep, 'require'):
+            if verify_requirements:
+                ep.require()
+            plugin = ep.resolve()
+        else:
+            plugin = ep.load()
         if invoke_on_load:
             obj = plugin(*invoke_args, **invoke_kwds)
         else:
@@ -210,7 +288,7 @@ class ExtensionManager(object):
         """
         if not self.extensions:
             # FIXME: Use a more specific exception class here.
-            raise RuntimeError('No %s extensions found' % self.namespace)
+            raise NoMatches('No %s extensions found' % self.namespace)
         response = []
         for e in self.extensions:
             self._invoke_one_plugin(response.append, func, e, args, kwds)
@@ -252,6 +330,14 @@ class ExtensionManager(object):
                 LOG.error('error calling %r: %s', e.name, err)
                 LOG.exception(err)
 
+    def items(self):
+        """
+        Return an iterator of tuples of the form (name, extension).
+
+        This is analogous to the Mapping.items() method.
+        """
+        return self._extensions_by_name.items()
+
     def __iter__(self):
         """Produce iterator for the manager.
 
@@ -267,9 +353,9 @@ class ExtensionManager(object):
         produces the :class:`Extension` instance with the
         specified name.
         """
-        if self._extensions_by_name is None:
-            d = {}
-            for e in self.extensions:
-                d[e.name] = e
-            self._extensions_by_name = d
         return self._extensions_by_name[name]
+
+    def __contains__(self, name):
+        """Return true if name is in list of enabled extensions.
+        """
+        return any(extension.name == name for extension in self.extensions)
